#---------------------------------------------------------------------------
# simbad.pl
# Bo Frese Rasmussen [ESO \ ST-ECF] 
#
# This library requires astroperl !
#---------------------------------------------------------------------------


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# simbad  -- Query Simbad for coordinates of named object.
#
# USAGE: 	( $ra, $dec ) = &simbad( $object_name );
#
# REMARKS:	Required variables : 
#			$SimbadHost, $SimbadPort, $SimbadUser, $SimbadPswd
#
#		Exits on error - or if objetc not found ! 
#		Error messages in HTML.
#---------------------------------------------------------------------------
sub simbad
{
    local ($object) = @_;
    local ($coord)  = "";
    local ($nvals,$simconn);

    print DEBUG "Connecting to SIMBAD ...\n" if $debug;
    $simconn = &simbad_connect($SimbadHost, $SimbadPort, $SimbadUser, 
				$SimbadPswd);
    
    if ( $simconn == -1 ) {
	print "<h2>ERROR connecting to SIMBAD</h2>" ;
	exit 1;
    }

    print DEBUG "Query SIMBAD (conn. $simconn) for $object ...\n" if $debug;
    $nvals = &simbad_query($simconn, $object, "");
    if ($nvals >= 1) {
	&simbad_retrieve($simconn, 1);
	$nvals = &simbad_findata($simconn, "J", "");
	if ($nvals >= 1) {
	    $coord = &simbad_getdata($simconn, 1);
	    print DEBUG "SIMBAD coord. = $coord \n" if $debug;
	    
	} else {
	    print "<h2>Coordinates for \"$object\" not in SIMBAD</h2>";
	}
    } else {
	print "<h2>Object \"$object\" not found in SIMBAD</h2>";
    }
    &simbad_disconnect($simconn);

    if ( ! $coord ) {
	print "Please go back and re-specify your query....<p>\n";
	&html_tail;
	exit 1;
    }

    ( $ra, $dec, $precision ) = $coord 
	    =~ /\s*([0-9\.]+)\s+([-+]?[0-9\.]+)\s+\((\w+)\)/;

    $ra *= 36000.0;
    $dec *= 36000.0;

    $ra_str  = &ed_pic( $ra / 15 ," 99 59 59.9");
    $dec_str = &ed_pic( $dec ,"+99 59 59.9");

    print "<h2>SIMBAD coordinates for $object : $ra_str, $dec_str</h2>";

    return ( $ra_str, $dec_str );
}

sub searchbox
{
    local( $radius_in, $ra_in, $dec_in ) = @_; 
    local( $radius, $ra, $dec, $ra_hi, $ra_lo, $dec_hi, $dec_lo );

    $radius = &tr_pic( &fix_coord($radius_in),"+99 59 59.9") * 100.0/2;
    if ( $dec_in && ! grep( /\.\./, $dec ) ) {
    $dec = &tr_pic(&fix_coord($dec_in),"+99 59 59.9")* 100.0;
    $dec_lo = $dec - $radius;
    $dec_hi = $dec + $radius;
    if ( $dec_lo < -324000000 ) {
	$dec_lo = -324000000;
	$no_ra_range = 1;
    }
    if ( $dec_hi > 324000000 ) {
	$dec_hi = 324000000;
	$no_ra_range = 1;
    }
    $dec_in = 
	   &ed_pic( $dec_lo/100.0,"+99 59 59.9")
		. ".." .
	   &ed_pic( $dec_hi/100.0,"+99 59 59.9");
    }
    if ( $ra_in && ! grep( /\.\./, $ra ) ) {
	if ( $no_ra_range ) {
	    $ra_in = "";
	} else {
	  $MAS = 3600000;
	  print DEBUG "RA radius berfore correction = $radius\n"
			    if $debug;
	  $radius = &asind(&sind($radius/$MAS)/&cosd($dec/$MAS))*$MAS;
	  print DEBUG "RA radius after correction = $radius\n"
			    if $debug;
	  $ra = &tr_pic(&fix_coord($ra_in)," 99 59 59.99")*150.0;
	  $ra_lo = $ra - $radius;
	  $ra_hi = $ra + $radius;
	  $ra_lo += 1296000000 if ($ra_lo < 0.0 );
	  $ra_hi -= 1296000000 if ($ra_hi > 1296000000 );

	  if ( $ra_lo < $ra_hi ) {
	     $ra_in = 
		&ed_pic($ra_lo/150.0," 99 59 59.99")
		  . ".." .  
		&ed_pic($ra_hi/150.0," 99 59 59.99");
	  } else {
	     $ra_in = ">=" .
		&ed_pic($ra_lo/150.0," 99 59 59.99")
		  . " | <=" .  
		&ed_pic($ra_hi/150.0," 99 59 59.99");

	  }
	}
    }
    return ($ra_in, $dec_in);
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# simbad2  -- Query Simbad for coordinates of named object.
#
# USAGE: 	( $ra, $dec, $err ) = &simbad( $object_name );
#
# REMARKS:	Required variables : 
#			$SimbadHost, $SimbadPort, $SimbadUser, $SimbadPswd
#
#		Error messages in $err
#---------------------------------------------------------------------------
sub simbad2
{
    local ($object) = @_;
    local ($coord)  = "";
    local ($err)    = "";
    local ($nvals,$simconn);

    print DEBUG "Connecting to SIMBAD ...\n" if $debug;
    $simconn = &simbad_connect($SimbadHost, $SimbadPort, $SimbadUser, 
				$SimbadPswd);
    
    if ( $simconn == -1 ) {
	return( "","","ERROR connecting to SIMBAD") ;
    }

    print DEBUG "Query SIMBAD (conn. $simconn) for $object ...\n" if $debug;
    $nvals = &simbad_query($simconn, $object, "");
    if ($nvals >= 1) {
	&simbad_retrieve($simconn, 1);
	$nvals = &simbad_findata($simconn, "J", "");
	if ($nvals >= 1) {
	    $coord = &simbad_getdata($simconn, 1);
	    print DEBUG "SIMBAD coord. = $coord \n" if $debug;
	    
	} else {
	    $err = "Coordinates for \"$object\" not in SIMBAD";
	}
    } else {
	$err = "Object \"$object\" not found in SIMBAD";
    }
    &simbad_disconnect($simconn);

    if ( $err ) {
	    return( "","",$err) ;
    }

    ( $ra, $dec, $precision ) = $coord 
	    =~ /\s*([0-9\.]+)\s+([-+]?[0-9\.]+)\s+\((\w+)\)/;

    $ra *= 36000.0;
    $dec *= 36000.0;

    $ra_str  = &ed_pic( $ra / 15 ," 99 59 59.9");
    $dec_str = &ed_pic( $dec ,"+99 59 59.9");

    return ( $ra_str, $dec_str, "" );
}


#-- Do not edit below this line --
1;
